@@ -280,6 +280,8 @@ namespace System.IO
                     finally 
                     {
                         StaticWatcherRunLoopManager.UnscheduleFromRunLoop(_eventStream);
+                        _eventStream.Close();
+                        _eventStream = null;
                     }
                 }
             }
@@ -435,7 +437,9 @@ namespace System.IO
                         // The base FileSystemWatcher does a match check against the relative path before combining with 
                         // the root dir; however, null is special cased to signify the root dir, so check if we should use that.
                         ReadOnlySpan<char> relativePath = ReadOnlySpan<char>.Empty;
-                        if (!path.Equals(_fullDirectory, StringComparison.OrdinalIgnoreCase))
+                        if (!path.Equals(_fullDirectory, StringComparison.OrdinalIgnoreCase)
+                            && path.Length >= _fullDirectory.Length
+                            && _fullDirectory.AsSpan().Equals(path.Slice(0, _fullDirectory.Length), StringComparison.OrdinalIgnoreCase))
                         {
                             // Remove the root directory to get the relative path
                             relativePath = path.Slice(_fullDirectory.Length);
@@ -479,9 +483,18 @@ namespace System.IO
                             else
                             {
                                 // Remove the base directory prefix and add the paired event to the list of 
-                                // events to skip and notify the user of the rename 
-                                ReadOnlySpan<char> newPathRelativeName = events[pairedId].Span.Slice(_fullDirectory.Length);
-                                watcher.NotifyRenameEventArgs(WatcherChangeTypes.Renamed, newPathRelativeName, relativePath);
+                                // events to skip and notify the user of the rename
+                                if (events[pairedId].Span.Length >= _fullDirectory.Length
+                                    && _fullDirectory.AsSpan().Equals(events[pairedId].Span.Slice(0, _fullDirectory.Length), StringComparison.OrdinalIgnoreCase))
+                                {
+                                    ReadOnlySpan<char> newPathRelativeName = events[pairedId].Span.Slice(_fullDirectory.Length);
+                                    watcher.NotifyRenameEventArgs(WatcherChangeTypes.Renamed, newPathRelativeName, relativePath);
+                                }
+                                else
+                                {
+                                    //if the base directory prefix isn't there, just use the full absolute path
+                                    watcher.NotifyRenameEventArgs(WatcherChangeTypes.Renamed, events[pairedId].Span, relativePath);
+                                }
 
                                 // Create a new list, if necessary, and add the event
                                 if (handledRenameEvents == null)
