@@ -212,6 +212,9 @@ typedef guint32 gunichar;
  */
 #define G_N_ELEMENTS(s)      (sizeof(s) / sizeof ((s) [0]))
 
+// e.g. strncmp (foo, G_STRING_CONSTANT_AND_LENGTH ("version"))
+#define G_STRING_CONSTANT_AND_LENGTH(x) (x), G_N_ELEMENTS (x) - 1
+
 #define FALSE                0
 #define TRUE                 1
 
@@ -375,6 +378,7 @@ gchar       *g_strchug        (gchar *str);
 gchar       *g_strchomp       (gchar *str);
 void         g_strdown        (gchar *string);
 gchar       *g_strnfill       (gsize length, gchar fill_char);
+gsize        g_strnlen        (const char*, gsize);
 
 void	     g_strdelimit     (char *string, char delimiter, char new_delimiter);
 gchar       *g_strescape      (const gchar *source, const gchar *exceptions);
@@ -1123,6 +1127,12 @@ gboolean   g_file_test (const gchar *filename, GFileTest test);
 #else
 #define g_write write
 #endif
+#ifdef G_OS_WIN32
+#define g_read _read
+#else
+#define g_read read
+#endif
+
 #define g_fopen fopen
 #define g_lstat lstat
 #define g_rmdir rmdir
@@ -1133,10 +1143,36 @@ gboolean   g_file_test (const gchar *filename, GFileTest test);
 
 gchar *g_mkdtemp (gchar *tmpl);
 
-
 /*
  * Low-level write-based printing functions
  */
+
+static inline int
+g_async_safe_fgets (char *str, int num, int handle, gboolean *newline)
+{
+	memset (str, 0, num);
+	// Make sure we don't overwrite the last index so that we are
+	// guaranteed to be NULL-terminated
+	int without_padding = num - 1;
+	int i=0;
+	while (i < without_padding && g_read (handle, &str [i], sizeof(char))) {
+		if (str [i] == '\n') {
+			str [i] = '\0';
+			*newline = TRUE;
+		}
+		
+		if (!isprint (str [i]))
+			str [i] = '\0';
+
+		if (str [i] == '\0')
+			break;
+
+		i++;
+	}
+
+	return i;
+}
+
 static inline gint
 g_async_safe_vfprintf (int handle, gchar const *format, va_list args)
 {
