@@ -28,7 +28,7 @@ mono_assembly_init_weak_fields (MonoImage *image);
 
 void
 mono_reflection_create_custom_attr_data_args_noalloc (MonoImage *image, MonoMethod *method, const guchar *data, guint32 len,
-													  gpointer **typed_args, gpointer **named_args, int *num_named_args,
+													  gpointer **typed_args_out, gpointer **named_args_out, int *num_named_args,
 													  CattrNamedArg **named_arg_info, MonoError *error);
 
 #endif  /* __MONO_METADATA_REFLECTION_CUSTOM_ATTRS_INTERNALS_H__ */
