@@ -121,14 +121,23 @@ static MonoFreeDomainFunc free_domain_hook;
 /* AOT cache configuration */
 static MonoAotCacheConfig aot_cache_config;
 
-static void
-get_runtimes_from_exe (const char *exe_file, MonoImage **exe_image, const MonoRuntimeInfo** runtimes);
+static GSList*
+get_runtimes_from_exe (const char *exe_file, MonoImage **exe_image);
 
 static const MonoRuntimeInfo*
 get_runtime_by_version (const char *version);
 
-MonoAssembly *
-mono_domain_assembly_open_internal (MonoDomain *domain, const char *name);
+static void
+mono_domain_alcs_destroy (MonoDomain *domain);
+
+static void
+mono_domain_alcs_lock (MonoDomain *domain);
+
+static void
+mono_domain_alcs_unlock (MonoDomain *domain);
+
+static void
+mono_domain_create_default_alc (MonoDomain *domain);
 
 static LockFreeMempool*
 lock_free_mempool_new (void)
@@ -457,6 +466,10 @@ mono_domain_create (void)
 	mono_os_mutex_init_recursive (&domain->jit_code_hash_lock);
 	mono_os_mutex_init_recursive (&domain->finalizable_objects_hash_lock);
 
+#ifdef ENABLE_NETCORE
+	mono_coop_mutex_init (&domain->alcs_lock);
+#endif
+
 	mono_appdomains_lock ();
 	domain_id_alloc (domain);
 	mono_appdomains_unlock ();
@@ -468,6 +481,10 @@ mono_domain_create (void)
 
 	mono_debug_domain_create (domain);
 
+#ifdef ENABLE_NETCORE
+	mono_domain_create_default_alc (domain);
+#endif
+
 	if (create_domain_hook)
 		create_domain_hook (domain);
 
@@ -495,8 +512,7 @@ mono_init_internal (const char *filename, const char *exe_filename, const char *
 	static MonoDomain *domain = NULL;
 	MonoAssembly *ass = NULL;
 	MonoImageOpenStatus status = MONO_IMAGE_OK;
-	const MonoRuntimeInfo* runtimes [G_N_ELEMENTS (supported_runtimes) + 1] = { NULL };
-	int n;
+	GSList *runtimes = NULL;
 
 #ifdef DEBUG_DOMAIN_UNLOAD
 	debug_domain_unload = TRUE;
@@ -563,36 +579,37 @@ mono_init_internal (const char *filename, const char *exe_filename, const char *
 		 * that would mean it would be reloaded later. So instead, we save it to
 		 * exe_image, and close it during shutdown.
 		 */
-		get_runtimes_from_exe (exe_filename, &exe_image, runtimes);
+		runtimes = get_runtimes_from_exe (exe_filename, &exe_image);
 #ifdef HOST_WIN32
 		if (!exe_image) {
-			exe_image = mono_assembly_open_from_bundle (exe_filename, NULL, FALSE);
+			exe_image = mono_assembly_open_from_bundle (mono_domain_default_alc (domain), exe_filename, NULL, FALSE);
 			if (!exe_image)
 				exe_image = mono_image_open (exe_filename, NULL);
 		}
 		mono_fixup_exe_image (exe_image);
 #endif
 	} else if (runtime_version != NULL) {
-		runtimes [0] = get_runtime_by_version (runtime_version);
-		runtimes [1] = NULL;
+		runtimes = g_slist_prepend (runtimes, (gpointer)get_runtime_by_version (runtime_version));
 	}
 
-	if (runtimes [0] == NULL) {
+	if (runtimes == NULL) {
 		const MonoRuntimeInfo *default_runtime = get_runtime_by_version (DEFAULT_RUNTIME_VERSION);
-		runtimes [0] = default_runtime;
-		runtimes [1] = NULL;
+		runtimes = g_slist_prepend (runtimes, (gpointer)default_runtime);
 		g_print ("WARNING: The runtime version supported by this application is unavailable.\n");
 		g_print ("Using default runtime: %s\n", default_runtime->runtime_version); 
 	}
 
 	/* The selected runtime will be the first one for which there is a mscrolib.dll */
-	for (n = 0; runtimes [n] != NULL && ass == NULL; n++) {
-		current_runtime = runtimes [n];
+	GSList *tmp = runtimes;
+	while (tmp != NULL) {
+		current_runtime = (MonoRuntimeInfo*)runtimes->data;
 		ass = mono_assembly_load_corlib (current_runtime, &status);
 		if (status != MONO_IMAGE_OK && status != MONO_IMAGE_ERROR_ERRNO)
 			break;
-
+		tmp = tmp->next;
 	}
+
+	g_slist_free (runtimes);
 	
 	if ((status != MONO_IMAGE_OK) || (ass == NULL)) {
 		switch (status){
@@ -1004,13 +1021,14 @@ mono_domain_assembly_open (MonoDomain *domain, const char *name)
 {
 	MonoAssembly *result;
 	MONO_ENTER_GC_UNSAFE;
-	result = mono_domain_assembly_open_internal (domain, name);
+	result = mono_domain_assembly_open_internal (domain, mono_domain_default_alc (domain), name);
 	MONO_EXIT_GC_UNSAFE;
 	return result;
 }
 
+// Uses the domain on legacy mono and the ALC on current
 MonoAssembly *
-mono_domain_assembly_open_internal (MonoDomain *domain, const char *name)
+mono_domain_assembly_open_internal (MonoDomain *domain, MonoAssemblyLoadContext *alc, const char *name)
 {
 	MonoDomain *current;
 	MonoAssembly *ass;
@@ -1018,6 +1036,17 @@ mono_domain_assembly_open_internal (MonoDomain *domain, const char *name)
 
 	MONO_REQ_GC_UNSAFE_MODE;
 
+#ifdef ENABLE_NETCORE
+	mono_alc_assemblies_lock (alc);
+	for (tmp = alc->loaded_assemblies; tmp; tmp = tmp->next) {
+		ass = (MonoAssembly *)tmp->data;
+		if (strcmp (name, ass->aname.name) == 0) {
+			mono_alc_assemblies_unlock (alc);
+			return ass;
+		}
+	}
+	mono_alc_assemblies_unlock (alc);
+#else
 	mono_domain_assemblies_lock (domain);
 	for (tmp = domain->domain_assemblies; tmp; tmp = tmp->next) {
 		ass = (MonoAssembly *)tmp->data;
@@ -1027,9 +1056,11 @@ mono_domain_assembly_open_internal (MonoDomain *domain, const char *name)
 		}
 	}
 	mono_domain_assemblies_unlock (domain);
+#endif
 
 	MonoAssemblyOpenRequest req;
 	mono_assembly_request_prepare (&req.request, sizeof (req), MONO_ASMCTX_DEFAULT);
+	req.request.alc = alc;
 	if (domain != mono_domain_get ()) {
 		current = mono_domain_get ();
 
@@ -1060,6 +1091,8 @@ unregister_vtable_reflection_type (MonoVTable *vtable)
  * This releases the resources associated with the specific domain.
  * This is a low-level function that is invoked by the AppDomain infrastructure
  * when necessary.
+ *
+ * In theory, this is dead code on netcore and thus does not need to be ALC-aware.
  */
 void
 mono_domain_free (MonoDomain *domain, gboolean force)
@@ -1240,6 +1273,12 @@ mono_domain_free (MonoDomain *domain, gboolean force)
 		domain->method_to_dyn_method = NULL;
 	}
 
+	mono_domain_alcs_destroy (domain);
+
+#ifdef ENABLE_NETCORE
+	mono_coop_mutex_destroy (&domain->alcs_lock);
+#endif
+
 	mono_os_mutex_destroy (&domain->finalizable_objects_hash_lock);
 	mono_os_mutex_destroy (&domain->assemblies_lock);
 	mono_os_mutex_destroy (&domain->jit_code_hash_lock);
@@ -1873,13 +1912,14 @@ get_runtime_by_version (const char *version)
 	return NULL;
 }
 
-static void
-get_runtimes_from_exe (const char *file, MonoImage **out_image, const MonoRuntimeInfo** runtimes)
+static GSList*
+get_runtimes_from_exe (const char *file, MonoImage **out_image)
 {
 	AppConfigInfo* app_config;
 	char *version;
 	const MonoRuntimeInfo* runtime = NULL;
 	MonoImage *image = NULL;
+	GSList *runtimes = NULL;
 	
 	app_config = app_config_parse (file);
 	
@@ -1888,32 +1928,32 @@ get_runtimes_from_exe (const char *file, MonoImage **out_image, const MonoRuntim
 		 * If there are no such elements, look for a requiredRuntime element.
 		 */
 		if (app_config->supported_runtimes != NULL) {
-			int n = 0;
 			GSList *list = app_config->supported_runtimes;
 			while (list != NULL) {
 				version = (char*) list->data;
 				runtime = get_runtime_by_version (version);
 				if (runtime != NULL)
-					runtimes [n++] = runtime;
+					runtimes = g_slist_prepend (runtimes, (gpointer)runtime);
 				list = g_slist_next (list);
 			}
-			runtimes [n] = NULL;
+			runtimes = g_slist_reverse (runtimes);
 			app_config_free (app_config);
-			return;
+			return runtimes;
 		}
 		
 		/* Check the requiredRuntime element. This is for 1.0 apps only. */
 		if (app_config->required_runtime != NULL) {
-			runtimes [0] = get_runtime_by_version (app_config->required_runtime);
-			runtimes [1] = NULL;
+			const MonoRuntimeInfo* runtime = get_runtime_by_version (app_config->required_runtime);
+			if (runtime != NULL)
+				runtimes = g_slist_prepend (runtimes, (gpointer)runtime);
 			app_config_free (app_config);
-			return;
+			return runtimes;
 		}
 		app_config_free (app_config);
 	}
 	
 	/* Look for a runtime with the exact version */
-	image = mono_assembly_open_from_bundle (file, NULL, FALSE);
+	image = mono_assembly_open_from_bundle (mono_domain_default_alc (mono_domain_get ()), file, NULL, FALSE);
 
 	if (image == NULL)
 		image = mono_image_open (file, NULL);
@@ -1923,15 +1963,17 @@ get_runtimes_from_exe (const char *file, MonoImage **out_image, const MonoRuntim
 		 * a default runtime and leave to the initialization method the work of
 		 * reporting the error.
 		 */
-		runtimes [0] = get_runtime_by_version (DEFAULT_RUNTIME_VERSION);
-		runtimes [1] = NULL;
-		return;
+		runtime = get_runtime_by_version (DEFAULT_RUNTIME_VERSION);
+		runtimes = g_slist_prepend (runtimes, (gpointer)runtime);
+		return runtimes;
 	}
 
 	*out_image = image;
 
-	runtimes [0] = get_runtime_by_version (image->version);
-	runtimes [1] = NULL;
+	runtime = get_runtime_by_version (image->version);
+	if (runtime != NULL)
+		runtimes = g_slist_prepend (runtimes, (gpointer)runtime);
+	return runtimes;
 }
 
 
@@ -2002,3 +2044,98 @@ mono_domain_get_assemblies (MonoDomain *domain, gboolean refonly)
 	mono_domain_assemblies_unlock (domain);
 	return assemblies;
 }
+
+MonoAssemblyLoadContext *
+mono_domain_default_alc (MonoDomain *domain)
+{
+#ifndef ENABLE_NETCORE
+	return NULL;
+#else
+	return domain->default_alc;
+#endif
+}
+
+#ifdef ENABLE_NETCORE
+static inline void
+mono_domain_alcs_lock (MonoDomain *domain)
+{
+	mono_coop_mutex_lock (&domain->alcs_lock);
+}
+
+static inline void
+mono_domain_alcs_unlock (MonoDomain *domain)
+{
+	mono_coop_mutex_unlock (&domain->alcs_lock);
+}
+#endif
+
+
+static MonoAssemblyLoadContext *
+create_alc (MonoDomain *domain, gboolean is_default)
+{
+#ifdef ENABLE_NETCORE
+	MonoAssemblyLoadContext *alc = NULL;
+
+	mono_domain_alcs_lock (domain);
+	if (is_default && domain->default_alc)
+		goto leave;
+
+	alc = g_new0 (MonoAssemblyLoadContext, 1);
+	mono_alc_init (alc, domain);
+
+	domain->alcs = g_slist_prepend (domain->alcs, alc);
+	if (is_default)
+		domain->default_alc = alc;
+leave:
+	mono_domain_alcs_unlock (domain);
+	return alc;
+#else
+	return NULL;
+#endif
+}
+
+void
+mono_domain_create_default_alc (MonoDomain *domain)
+{
+#ifdef ENABLE_NETCORE
+	if (domain->default_alc)
+		return;
+	create_alc (domain, TRUE);
+#endif
+}
+
+#ifdef ENABLE_NETCORE
+MonoAssemblyLoadContext *
+mono_domain_create_individual_alc (MonoDomain *domain, uint32_t this_gchandle, gboolean collectible, MonoError *error)
+{
+	g_assert (!collectible); /* TODO: implement collectible ALCs */
+	MonoAssemblyLoadContext *alc = create_alc (domain, FALSE);
+	alc->gchandle = this_gchandle;
+	return alc;
+}
+#endif
+
+static void
+mono_alc_free (MonoAssemblyLoadContext *alc)
+{
+#ifdef ENABLE_NETCORE
+	mono_alc_cleanup (alc);
+	g_free (alc);
+#endif
+}
+
+void
+mono_domain_alcs_destroy (MonoDomain *domain)
+{
+#ifdef ENABLE_NETCORE
+	mono_domain_alcs_lock (domain);
+	GSList *alcs = domain->alcs;
+	domain->alcs = NULL;
+	domain->default_alc = NULL;
+	mono_domain_alcs_unlock (domain);
+
+	for (GSList *iter = alcs; iter; iter = g_slist_next (iter)) {
+		mono_alc_free ((MonoAssemblyLoadContext*)iter->data);
+	}
+#endif
+}
