@@ -234,11 +234,6 @@ mono_gc_is_critical_method (MonoMethod *method)
 #endif
 }
 
-static void
-emit_nursery_check_noilgen (MonoMethodBuilder *mb, gboolean is_concurrent)
-{
-}
-
 static MonoSgenMonoCallbacks sgenmono_cb;
 static gboolean cb_inited = FALSE;
 
@@ -251,6 +246,13 @@ mono_install_sgen_mono_callbacks (MonoSgenMonoCallbacks *cb)
 	cb_inited = TRUE;
 }
 
+#if !ENABLE_ILGEN
+
+static void
+emit_nursery_check_noilgen (MonoMethodBuilder *mb, gboolean is_concurrent)
+{
+}
+
 static void
 emit_managed_allocater_noilgen (MonoMethodBuilder *mb, gboolean slowpath, gboolean profiler, int atype)
 {
@@ -266,6 +268,8 @@ install_noilgen (void)
 	mono_install_sgen_mono_callbacks (&cb);
 }
 
+#endif
+
 static MonoSgenMonoCallbacks *
 get_sgen_mono_cb (void)
 {
@@ -2119,6 +2123,7 @@ sgen_client_thread_register_worker (void)
 {
 	mono_thread_info_register_small_id ();
 	mono_native_thread_set_name (mono_native_thread_id_get (), "SGen worker");
+	mono_thread_set_name_windows (GetCurrentThread (), L"SGen worker");
 }
 
 /* Variables holding start/end nursery so it won't have to be passed at every call */
@@ -2153,6 +2158,9 @@ pin_handle_stack_interior_ptrs (void **ptr_slot, void *user_data)
 	sgen_conservatively_pin_objects_from (ptr_slot, ptr_slot+1, ud->start_nursery, ud->end_nursery, PIN_TYPE_STACK);
 }
 
+#ifdef HOST_WASM
+extern gboolean mono_wasm_enable_gc;
+#endif
 
 /*
  * Mark from thread stacks and registers.
@@ -2164,7 +2172,8 @@ sgen_client_scan_thread_data (void *start_nursery, void *end_nursery, gboolean p
 	scan_area_arg_end = end_nursery;
 #ifdef HOST_WASM
 	//Under WASM we don't scan thread stacks and we can't trust the values we find there either.
-	return;
+	if (!mono_wasm_enable_gc)
+		return;
 #endif
 
 	FOREACH_THREAD_EXCLUDE (info, MONO_THREAD_INFO_FLAGS_NO_GC) {
@@ -2486,6 +2495,16 @@ mono_gc_get_los_limit (void)
 	return SGEN_MAX_SMALL_OBJ_SIZE;
 }
 
+guint64
+mono_gc_get_allocated_bytes_for_current_thread (void)
+{
+	SgenThreadInfo* info;
+	info = mono_thread_info_current ();
+
+	/*There are some more allocated bytes in the current tlab that have not been recorded yet */
+	return info->total_bytes_allocated + info->tlab_next - info->tlab_start;
+}
+
 gpointer
 sgen_client_default_metadata (void)
 {
