@@ -34,6 +34,7 @@
 
 typedef struct {
 	gboolean (*init) (gint wakeup_pipe_fd);
+	gboolean (*can_register_fd) (int fd);
 	void     (*register_fd) (gint fd, gint events, gboolean is_new);
 	void     (*remove_fd) (gint fd);
 	gint     (*event_wait) (void (*callback) (gint fd, gint events, gpointer user_data), gpointer user_data);
@@ -330,7 +331,7 @@ selector_thread (gpointer data)
 
 	MonoString *thread_name = mono_string_new_checked (mono_get_root_domain (), "Thread Pool I/O Selector", error);
 	mono_error_assert_ok (error);
-	mono_thread_set_name_internal (mono_thread_internal_current (), thread_name, FALSE, TRUE, error);
+	mono_thread_set_name_internal (mono_thread_internal_current (), thread_name, MonoSetThreadNameFlag_Reset, error);
 	mono_error_assert_ok (error);
 
 	if (mono_runtime_is_shutting_down ()) {
@@ -603,6 +604,7 @@ mono_threadpool_io_cleanup (void)
 void
 ves_icall_System_IOSelector_Add (gpointer handle, MonoIOSelectorJob *job)
 {
+	ERROR_DECL (error);
 	ThreadPoolIOUpdate *update;
 
 	g_assert (handle);
@@ -624,9 +626,19 @@ ves_icall_System_IOSelector_Add (gpointer handle, MonoIOSelectorJob *job)
 		return;
 	}
 
+	int fd = GPOINTER_TO_INT (handle);
+
+	if (!threadpool_io->backend.can_register_fd (fd)) {
+		mono_coop_mutex_unlock (&threadpool_io->updates_lock);
+		mono_trace (G_LOG_LEVEL_WARNING, MONO_TRACE_IO_SELECTOR, "Could not register to wait for file descriptor %d", fd);
+		mono_error_set_not_supported (error, "Could not register to wait for file descriptor %d", fd);
+		mono_error_set_pending_exception (error);
+		return;
+	}
+
 	update = update_get_new ();
 	update->type = UPDATE_ADD;
-	update->data.add.fd = GPOINTER_TO_INT (handle);
+	update->data.add.fd = fd;
 	update->data.add.job = job;
 	mono_memory_barrier (); /* Ensure this is safely published before we wake up the selector */
 
@@ -658,7 +670,7 @@ mono_threadpool_io_remove_socket (int fd)
 
 	update = update_get_new ();
 	update->type = UPDATE_REMOVE_SOCKET;
-	update->data.add.fd = fd;
+	update->data.remove_socket.fd = fd;
 	mono_memory_barrier (); /* Ensure this is safely published before we wake up the selector */
 
 	selector_thread_wakeup ();
@@ -727,4 +739,4 @@ mono_threadpool_io_remove_domain_jobs (MonoDomain *domain)
 	g_assert_not_reached ();
 }
 
-#endif
\ No newline at end of file
+#endif
