@@ -21,13 +21,32 @@
 static const gboolean debug_tailcall_break_compile = FALSE; // break in method_to_ir
 static const gboolean debug_tailcall_break_run = FALSE;     // insert breakpoint in generated code
 
+MonoJumpInfoTarget
+mono_call_to_patch (MonoCallInst *call)
+{
+	MonoJumpInfoTarget patch;
+	MonoJitICallId jit_icall_id;
+
+	// This is similar to amd64 emit_call.
+
+	if (call->inst.flags & MONO_INST_HAS_METHOD) {
+		patch.type = MONO_PATCH_INFO_METHOD;
+		patch.target = call->method;
+	} else if ((jit_icall_id = call->jit_icall_id)) {
+		patch.type = MONO_PATCH_INFO_JIT_ICALL_ID;
+		patch.target = GUINT_TO_POINTER (jit_icall_id);
+	} else {
+		patch.type = MONO_PATCH_INFO_ABS;
+		patch.target = call->fptr;
+	}
+	return patch;
+}
+
 void
 mono_call_add_patch_info (MonoCompile *cfg, MonoCallInst *call, int ip)
 {
-	if (call->inst.flags & MONO_INST_HAS_METHOD)
-		mono_add_patch_info (cfg, ip, MONO_PATCH_INFO_METHOD, call->method);
-	else
-		mono_add_patch_info (cfg, ip, MONO_PATCH_INFO_ABS, call->fptr);
+	const MonoJumpInfoTarget patch = mono_call_to_patch (call);
+	mono_add_patch_info (cfg, ip, patch.type, patch.target);
 }
 
 void
@@ -38,7 +57,7 @@ mini_test_tailcall (MonoCompile *cfg, gboolean tailcall)
 	//
 	// Do not change "tailcalllog" here without changing other places, e.g. tests that search for it.
 	//
-	g_assertf (tailcall || !mini_get_debug_options ()->test_tailcall_require, "tailcalllog fail from %s", cfg->method->name);
+	g_assertf (tailcall || !mini_debug_options.test_tailcall_require, "tailcalllog fail from %s", cfg->method->name);
 	mono_tailcall_print ("tailcalllog %s from %s\n", tailcall ? "success" : "fail", cfg->method->name);
 }
 
@@ -613,6 +632,7 @@ mono_emit_method_call (MonoCompile *cfg, MonoMethod *method, MonoInst **args, Mo
 	return mini_emit_method_call_full (cfg, method, mono_method_signature_internal (method), FALSE, args, this_ins, NULL, NULL);
 }
 
+static
 MonoInst*
 mono_emit_native_call (MonoCompile *cfg, gconstpointer func, MonoMethodSignature *sig,
 					   MonoInst **args)
@@ -629,22 +649,16 @@ mono_emit_native_call (MonoCompile *cfg, gconstpointer func, MonoMethodSignature
 	return (MonoInst*)call;
 }
 
-// FiXME remove or make static
-MonoInst*
-mono_emit_jit_icall_info (MonoCompile *cfg, MonoJitICallInfo *info, MonoInst **args)
-{
-	g_assert (info);
-	g_assertf (info->name, "%d", (int)mono_jit_icall_info_index (info));
-	g_assertf (info->func, "%d", (int)mono_jit_icall_info_index (info));
-
-	return mono_emit_native_call (cfg, mono_icall_get_wrapper (info), info->sig, args);
-}
-
-
 MonoInst*
 mono_emit_jit_icall_id (MonoCompile *cfg, MonoJitICallId jit_icall_id, MonoInst **args)
 {
-	return mono_emit_jit_icall_info (cfg, mono_find_jit_icall_info (jit_icall_id), args);
+	MonoJitICallInfo *info = mono_find_jit_icall_info (jit_icall_id);
+
+	MonoCallInst *call = (MonoCallInst *)mono_emit_native_call (cfg, mono_icall_get_wrapper (info), info->sig, args);
+
+	call->jit_icall_id = jit_icall_id;
+
+	return (MonoInst*)call;
 }
 
 /*
@@ -662,6 +676,8 @@ mini_emit_abs_call (MonoCompile *cfg, MonoJumpInfoType patch_type, gconstpointer
 	/* 
 	 * We pass ji as the call address, the PATCH_INFO_ABS resolving code will
 	 * handle it.
+	 * FIXME: Is the abs_patches hashtable avoidable?
+	 * Such as by putting the patch info in the call instruction?
 	 */
 	if (cfg->abs_patches == NULL)
 		cfg->abs_patches = g_hash_table_new (NULL, NULL);
