@@ -351,13 +351,12 @@ mono_de_add_pending_breakpoints (MonoMethod *method, MonoJitInfo *ji)
 static void
 set_bp_in_method (MonoDomain *domain, MonoMethod *method, MonoSeqPointInfo *seq_points, MonoBreakpoint *bp, MonoError *error)
 {
-	gpointer code;
 	MonoJitInfo *ji;
 
 	if (error)
 		error_init (error);
 
-	code = mono_jit_search_all_backends_for_jit_info (domain, method, &ji);
+	(void)mono_jit_search_all_backends_for_jit_info (domain, method, &ji);
 	g_assert (ji);
 
 	insert_breakpoint (seq_points, domain, ji, bp, error);
@@ -1278,6 +1277,10 @@ mono_de_ss_start (SingleStepReq *ss_req, SingleStepArgs *ss_args)
 	int nframes = ss_args->nframes;
 	SeqPoint *sp = &ss_args->sp;
 
+	/* this can happen on a single step in a exception on android (Mono_UnhandledException_internal) and on IOS */
+	if (!method)
+		return;
+
 	/*
 	 * Implement single stepping using breakpoints if possible.
 	 */
