@@ -4,7 +4,7 @@ using System.Collections.ObjectModel;
 using System.Runtime.CompilerServices;
 using System.Threading.Tasks;
 
-struct Foo {
+public struct Foo {
 	public int i, j, k, l, m, n;
 }
 
@@ -1107,7 +1107,7 @@ public class Tests
 	interface IConstrainedCalls {
 		Pair<int, int> vtype_ret<T, T2>(T t, T2 t2) where T: IReturnVType;
 		AnEnum enum_ret<T, T2>(T t, T2 t2) where T: IReturnVType;
-		int normal_args<T, T2> (T t, T2 t2, int i1, int i2, string s, ref int i3) where T : IConstrained2;
+		int normal_args<T, T2> (T t, T2 t2, int i1, int i2, string s, ref int i3, Foo foo) where T : IConstrained2;
 	}
 
 	public interface IReturnVType {
@@ -1116,7 +1116,7 @@ public class Tests
 	}
 
 	public interface IConstrained2 {
-		int normal_args (int i1, int i2, string s, ref int i3);
+		int normal_args (int i1, int i2, string s, ref int i3, Foo foo);
 	}
 
 	public class CConstrainedCalls : IConstrainedCalls {
@@ -1130,8 +1130,8 @@ public class Tests
 			return t.return_enum ();
 		}
 
-		public int normal_args<T, T2> (T t, T2 t2, int i1, int i2, string s, ref int i3) where T : IConstrained2 {
-			return t.normal_args (i1, i2, s, ref i3);
+		public int normal_args<T, T2> (T t, T2 t2, int i1, int i2, string s, ref int i3, Foo foo) where T : IConstrained2 {
+			return t.normal_args (i1, i2, s, ref i3, foo);
 		}
 	}
 
@@ -1145,9 +1145,9 @@ public class Tests
 	}
 
 	class ConstrainedCalls : IConstrained2 {
-		public int normal_args (int i1, int i2, string s, ref int i3) {
+		public int normal_args (int i1, int i2, string s, ref int i3, Foo foo) {
 			i3 = i3 + 1;
-			return i1 + i2 + i3 + s.Length;
+			return i1 + i2 + i3 + s.Length + foo.i + foo.n;
 		}
 	}
 
@@ -1167,11 +1167,12 @@ public class Tests
 		return 0;
 	}
 
-	public static int test_14_constrained_normal_args () {
+	public static int test_25_constrained_normal_args () {
 		IConstrainedCalls c = new CConstrainedCalls ();
 
+		Foo foo = new Foo () { i = 5, n = 6 };
 		int val = 3;
-		var r = c.normal_args<ConstrainedCalls, int> (new ConstrainedCalls (), 0, 1, 2, "ABC", ref val);
+		var r = c.normal_args<ConstrainedCalls, int> (new ConstrainedCalls (), 0, 1, 2, "ABC", ref val, foo);
 		return r + val;
 	}
 
@@ -2191,6 +2192,30 @@ public class Tests
 		var s = new AStruct () { a = 1, b = 2 };
 		return iface.foo<AStruct> (s);
 	}
+
+	interface IFaceOpenDel {
+		object AMethod<T> ();
+	}
+
+	class ClassOpenDel : IFaceOpenDel {
+		public Nullable<int> field;
+
+		public Nullable<int> getField () {
+			return field;
+		}
+
+		public object AMethod<T> () {
+			var d = (Func<ClassOpenDel, T>)Delegate.CreateDelegate (typeof (Func<ClassOpenDel, T>), typeof (ClassOpenDel).GetMethod ("getField"));
+			return d (this);
+		}
+	}
+
+	// Open instance delegate returning a gsharedvt value
+	public static int test_0_open_delegate () {
+		IFaceOpenDel iface = new ClassOpenDel () { field = 42 };
+		var res = (Nullable<int>)iface.AMethod<Nullable<int>> ();
+		return res == 42 ? 0 : 1;
+	}
 }
 
 // #13191
