@@ -42,7 +42,7 @@
 #include <utils/mono-threads-debug.h>
 
 static const char *
-os_version_string (void)
+kernel_version_string (void)
 {
 #ifdef HAVE_SYS_UTSNAME_H
 	static struct utsname name;
@@ -62,6 +62,61 @@ os_version_string (void)
 	return "";
 }
 
+static gboolean
+starts_with (const char *pre, size_t pre_sz, const char *str)
+{
+    return strncmp (pre, str, pre_sz) == 0;
+}
+
+static const char *
+macos_version_string (void)
+{
+	/* Can't allocate in here, could be called from a signal handler in a
+	 * crashed process.
+	 */
+	static const char *version_string;
+	static char buf[256];
+	static const size_t buf_size = sizeof (buf);
+
+	if (version_string)
+		return version_string;
+
+	/* macOS 10.13.6 or later */
+	if (!version_string) {
+		size_t size = 0;
+		if (sysctlbyname ("kern.osproductversion", NULL, &size, NULL, 0) < 0 || size >= buf_size) {
+			/* if we couldn't get the size or if it needs more space that we have in buf, leave it empty */
+			version_string = "";
+			return version_string;
+		}
+
+		if (sysctlbyname ("kern.osproductversion", (char*)buf, &size, NULL, 0) >= 0)
+			version_string = &buf[0];
+	}
+	/* macOS 10.13.5 or older */
+	if (!version_string) {
+		const char *kv_string = kernel_version_string ();
+		if (starts_with (G_STRING_CONSTANT_AND_LENGTH ("17"), kv_string))
+			version_string = "10.13"; // High Sierra
+		else if (starts_with (G_STRING_CONSTANT_AND_LENGTH ("16"), kv_string))
+			version_string = "10.12"; // Sierra
+		else if (starts_with (G_STRING_CONSTANT_AND_LENGTH ("15"), kv_string))
+			version_string = "10.11"; // El Capitan
+		else if (starts_with (G_STRING_CONSTANT_AND_LENGTH ("14"), kv_string))
+			version_string = "10.10"; // Yosemite
+		else if (starts_with (G_STRING_CONSTANT_AND_LENGTH ("13"), kv_string))
+			version_string = "10.9"; // Mavericks
+		else if (starts_with (G_STRING_CONSTANT_AND_LENGTH ("12"), kv_string))
+			version_string = "10.8"; // Mountain Lion
+		else if (starts_with (G_STRING_CONSTANT_AND_LENGTH ("11"), kv_string))
+			version_string = "10.7"; // Lion
+	}
+	if (!version_string)
+		version_string = "";
+
+	return version_string;
+}
+
 // To get the path of the running process
 #include <libproc.h>
 
@@ -114,12 +169,18 @@ typedef struct {
 	char systemModel [100];
 	const char *systemManufacturer;
 
-	const char *eventType;
+	const char *eventType; /* Must be MONO_MERP_EVENT_TYPE_STR */
 
 	MonoStackHash hashes;
 	GSList *annotations;
 } MERPStruct;
 
+/* The event type determines the format of the fields that are reported.  It
+ * must be MonoAppCrash for the rest of our report to make sense.
+ */
+#define MONO_MERP_EVENT_TYPE_STR "MonoAppCrash"
+
+
 typedef struct {
 	gboolean enable_merp;
 
@@ -280,8 +341,6 @@ mono_merp_send (MERPStruct *merp)
 		exit (-1);
 	} else {
 		int status;
-		waitpid (pid, &status, 0);
-		gboolean exit_success = FALSE;
 		int exit_status = FALSE;
 
 		while (TRUE) {
@@ -290,8 +349,7 @@ mono_merp_send (MERPStruct *merp)
 
 			if (WIFEXITED(status)) {
 				exit_status = WEXITSTATUS(status);
-				exit_success = TRUE;
-				invoke_success = exit_status == TRUE;
+				invoke_success = (exit_status == 0);
 				break;
 			} else if (WIFSIGNALED(status)) {
 				break;
@@ -350,14 +408,17 @@ mono_init_merp (const intptr_t crashed_pid, const char *signal, MonoStackHash *h
 	merp->moduleOffset = 0;
 
 	merp->uiLidArg = MONO_LOCALE_INVARIANT;
-
-	merp->osVersion = os_version_string ();
+#if defined (TARGET_OSX)
+	merp->osVersion = macos_version_string ();
+#else
+	merp->osVersion = kernel_version_string ();
+#endif
 
 	// FIXME: THis is apple-only for now
 	merp->systemManufacturer = "apple";
 	get_apple_model ((char *) merp->systemModel, sizeof (merp->systemModel));
 
-	merp->eventType = config.eventType;
+	merp->eventType = MONO_MERP_EVENT_TYPE_STR;
 
 	merp->hashes = *hashes;
 
@@ -372,7 +433,7 @@ mono_merp_write_fingerprint_payload (const char *non_param_data, const MERPStruc
 
 	g_async_safe_fprintf(handle, "{\n");
 	g_async_safe_fprintf(handle, "\t\"payload\" : \n");
-	g_write (handle, non_param_data, (guint32)strlen (non_param_data));	\
+	g_write (handle, non_param_data, (guint32)strlen (non_param_data));
 	g_async_safe_fprintf(handle, ",\n");
 
 	g_async_safe_fprintf(handle, "\t\"parameters\" : \n{\n");
@@ -526,7 +587,6 @@ mono_merp_disable (void)
 	g_free ((char*)config.appSignature);
 	g_free ((char*)config.appVersion);
 	g_free ((char*)config.merpGUIPath);
-	g_free ((char*)config.eventType);
 	g_free ((char*)config.appPath); 
 	g_free ((char*)config.moduleVersion);
 	g_slist_free (config.annotations);
@@ -536,7 +596,7 @@ mono_merp_disable (void)
 }
 
 void
-mono_merp_enable (const char *appBundleID, const char *appSignature, const char *appVersion, const char *merpGUIPath, const char *eventType, const char *appPath, const char *configDir)
+mono_merp_enable (const char *appBundleID, const char *appSignature, const char *appVersion, const char *merpGUIPath, const char *appPath, const char *configDir)
 {
 	mono_memory_barrier ();
 
@@ -561,7 +621,6 @@ mono_merp_enable (const char *appBundleID, const char *appSignature, const char
 	config.appSignature = g_strdup (appSignature);
 	config.appVersion = g_strdup (appVersion);
 	config.merpGUIPath = g_strdup (merpGUIPath);
-	config.eventType = g_strdup (eventType);
 	config.appPath = g_strdup (appPath);
 
 	config.log = g_getenv ("MONO_MERP_VERBOSE") != NULL;
