@@ -8,13 +8,16 @@
  * (C) 2018 Microsoft, Inc.
  *
  */
-#ifndef DISABLE_CRASH_REPORTING
-
 #include <config.h>
 #include <glib.h>
 #include <mono/utils/mono-state.h>
+#include <mono/utils/atomic.h>
+
+#ifndef DISABLE_CRASH_REPORTING
+
 #include <mono/utils/mono-threads-coop.h>
 #include <mono/metadata/object-internals.h>
+#include <mono/metadata/mono-config-dirs.h>
 
 #include <sys/param.h>
 #include <fcntl.h>
@@ -135,6 +138,15 @@ file_for_summary_stage (const char *directory, MonoSummaryStage stage, gchar *bu
 	g_snprintf (buff, sizeof_buff, "%s%scrash_stage_%d", directory, G_DIR_SEPARATOR_S, stage);
 }
 
+static void
+create_stage_mark_file (void)
+{
+	char out_file [200];
+	file_for_summary_stage (log.directory, log.level, out_file, sizeof(out_file));
+	int handle = g_open (out_file, O_WRONLY | O_CREAT, S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH);
+	close(handle);
+}
+
 gboolean
 mono_summarize_set_timeline_dir (const char *directory)
 {
@@ -155,30 +167,27 @@ mono_summarize_timeline_start (void)
 	if (!configured_timeline_dir)
 		return;
 
-	log.level = MonoSummarySetup;
 	log.directory = configured_timeline_dir;
+	mono_summarize_timeline_phase_log (MonoSummarySetup);
 }
 
 void
 mono_summarize_double_fault_log (void)
 {
-	char out_file [200];
-	file_for_summary_stage (log.directory, MonoSummaryDoubleFault, out_file, sizeof(out_file));
-	int handle = g_open (out_file, O_WRONLY | O_CREAT, S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH);
-	close(handle);
+	mono_summarize_timeline_phase_log (MonoSummaryDoubleFault);
 }
 
 void
 mono_summarize_timeline_phase_log (MonoSummaryStage next)
 {
-	if (log.level == MonoSummaryNone)
-		return;
-
 	if (!log.directory)
 		return;
 
 	MonoSummaryStage out_level;
 	switch (log.level) {
+		case MonoSummaryNone:
+			out_level = MonoSummarySetup;
+			break;
 		case MonoSummarySetup:
 			out_level = MonoSummarySuspendHandshake;
 			break;
@@ -222,23 +231,16 @@ mono_summarize_timeline_phase_log (MonoSummaryStage next)
 			return;
 	}
 
-	g_assertf(out_level == next, "Log Error: Log transition to %d, actual expected next step is %d\n", next, out_level);
-
-	char out_file [200];
-	memset (out_file, 0, sizeof(out_file));
-	file_for_summary_stage (log.directory, out_level, out_file, sizeof(out_file));
-
-	int handle = g_open (out_file, O_WRONLY | O_CREAT, S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH);
-	close(handle);
+	g_assertf(out_level == next || next == MonoSummaryDoubleFault, "Log Error: Log transition to %d, actual expected next step is %d\n", next, out_level);
 
+	log.level = out_level;
+	create_stage_mark_file ();
 	// To check, comment out normally
 	// DO NOT MERGE UNCOMMENTED
 	// As this does a lot of FILE io
 	//
 	// g_assert (out_level == mono_summarize_timeline_read_level (log.directory,  FALSE));
 
-	log.level = out_level;
-
 	if (out_level == MonoSummaryDone)
 		memset (&log, 0, sizeof (log));
 
@@ -441,12 +443,10 @@ mono_native_state_add_frame (MonoStateWriter *writer, MonoFrameSummary *frame)
 	mono_state_writer_printf(writer, "{\n");
 	writer->indent++;
 
-	if (frame->is_managed) {
-		assert_has_space (writer);
-		mono_state_writer_indent (writer);
-		mono_state_writer_object_key (writer, "is_managed");
-		mono_state_writer_printf(writer, "\"%s\",", frame->is_managed ? "true" : "false");
-	}
+	assert_has_space (writer);
+	mono_state_writer_indent (writer);
+	mono_state_writer_object_key (writer, "is_managed");
+	mono_state_writer_printf(writer, "\"%s\",", frame->is_managed ? "true" : "false");
 
 	if (frame->unmanaged_data.is_trampoline) {
 		mono_state_writer_printf(writer, "\n");
@@ -503,6 +503,7 @@ mono_native_state_add_frame (MonoStateWriter *writer, MonoFrameSummary *frame)
 		mono_state_writer_printf(writer, "\"0x%05x\"\n", frame->managed_data.il_offset);
 
 	} else {
+		mono_state_writer_printf(writer, "\n");
 		assert_has_space (writer);
 		mono_state_writer_indent (writer);
 		mono_state_writer_object_key (writer, "native_address");
@@ -520,7 +521,7 @@ mono_native_state_add_frame (MonoStateWriter *writer, MonoFrameSummary *frame)
 			mono_state_writer_printf(writer, "\"0x%05x\"", frame->unmanaged_data.offset);
 		}
 
-		if (frame->unmanaged_data.module) {
+		if (frame->unmanaged_data.module [0] != '\0') {
 			mono_state_writer_printf(writer, ",\n");
 
 			assert_has_space (writer);
@@ -927,6 +928,67 @@ mono_native_state_add_memory (MonoStateWriter *writer)
 	mono_state_writer_printf(writer, "},\n");
 }
 
+#define MONO_CRASH_REPORTING_MAPPING_LINE_LIMIT 30
+
+#if !MONO_PRIVATE_CRASHES
+
+static void
+mono_native_state_add_process_map (MonoStateWriter *writer)
+{
+#if defined(__linux__) && !defined(HOST_ANDROID)
+	int handle = g_open ("/proc/self/maps", O_RDONLY, S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH);
+	if (handle == -1) {
+		g_async_safe_printf ("Couldn't find /proc/self/maps on Linux system. Continuing.");
+		return;
+	}
+
+	assert_has_space (writer);
+	mono_state_writer_indent (writer);
+	mono_state_writer_object_key (writer, "process_map");
+	mono_state_writer_printf(writer, "[\n");
+
+	int mapping = 0;
+	while (mapping < MONO_CRASH_REPORTING_MAPPING_LINE_LIMIT) {
+		if (mapping > 0)
+			mono_state_writer_printf (writer, "\",\n");
+
+		mono_state_writer_printf (writer, "\t\"");
+
+		while (TRUE) {
+			char line [10];
+			gboolean newline = FALSE;
+			int charsCopied = g_async_safe_fgets (line, sizeof (line), handle, &newline);
+
+			if (charsCopied == 0)
+				break;
+
+			for (int i=0; i < charsCopied; i++)
+				g_assert (isprint (line [i]));
+
+			g_assert (line [charsCopied] == '\0');
+
+			mono_state_writer_printf (writer, "%s", line);
+
+			if (newline)
+				break;
+		}
+
+		mapping++;
+	}
+
+	if (mapping > 0)
+		mono_state_writer_printf (writer, "\"");
+
+	mono_state_writer_indent (writer);
+	writer->indent--;
+	mono_state_writer_printf(writer, "],\n");
+
+	close (handle);
+#endif
+}
+
+#endif
+
 static void
 mono_native_state_add_prologue (MonoStateWriter *writer)
 {
@@ -961,6 +1023,10 @@ mono_native_state_add_prologue (MonoStateWriter *writer)
 		mono_state_writer_printf(writer, "\"%.*s\",\n", (int)length, assertion_msg);
 	}
 
+#ifndef MONO_PRIVATE_CRASHES
+	mono_native_state_add_process_map (writer);
+#endif
+
 	// Start threads array
 	assert_has_space (writer);
 	mono_state_writer_indent (writer);
@@ -1076,3 +1142,17 @@ mono_crash_dump (const char *jsonFile, MonoStackHash *hashes)
 }
 
 #endif // DISABLE_CRASH_REPORTING
+
+static volatile int32_t dump_status;
+
+gboolean
+mono_dump_start (void)
+{
+	return (mono_atomic_xchg_i32(&dump_status, 1) == 0);  // return true if we started the dump
+}
+
+gboolean
+mono_dump_complete (void)
+{
+	return (mono_atomic_xchg_i32(&dump_status, 0) == 1);  // return true if we completed the dump
+}
